local NUM_TRADING_SLOTS = 7
local inventoryIndex = 0
local inventory = {}
local interfaceItem = {}
local interfaceIndex = 0
local color = "|cFF4FF4A2TradeFill:|r"

local TradeFill = LibStub("AceAddon-3.0"):NewAddon("MyAddon", "AceConsole-3.0", "AceEvent-3.0")

local classes = { "WARRIOR", "PALADIN", "HUNTER", "ROGUE", "PRIEST", "SHAMAN", "MAGE", "WARLOCK", "DRUID" }

local OptionFrame = CreateFrame("Frame", "MyIconButtonFrame", TradeFrame, "BackdropTemplate")
local ButtonA = CreateFrame("Button", "MyTradeButton", OptionFrame, "UIPanelButtonTemplate")
local ButtonB = CreateFrame("Button", "MyTradeButton", OptionFrame, "UIPanelButtonTemplate")
local ClearTrade = CreateFrame("Button", "MyTradeButton", OptionFrame, "UIPanelButtonTemplate")

local options = {
    name = "TradeFill",
    handler = TradeFill,
    type = "group",
    childGroups = "tab",
    args = {
        menu = {
            type = "group",
            name = "",
            order = 0,
            args = {
                toggleBoxA = {
                    type = 'toggle',
                    name = "Show only full stacks",
                    desc = "Hide stacks that are not fill in dropdown menu",
                    get = function(info) return TradeFill.itemdb.profile.show end,
                    set = function(info, value) TradeFill.itemdb.profile.show = value end,
                    order = 0,
                },
                separatorA = {
                    type = "description",
                    name = " ",
                    order = 1,
                },
                iconA = {
                    type = "description",
                    fontSize = "medium",
                    name = function(info) return TradeFill:GetName("A") end,
                    image = function(info) return TradeFill:SetIconImage("A") end,
                    imageWidth = 32,
                    imageHeight = 32,
                    order = 2,
                },
                itemA = {
                    type = "select",
                    width = 1.5,
                    name = "Item to trade",
                    desc = "Select a item",
                    values = function() return TradeFill:SetOptionItem("A") end,
                    get = function() return TradeFill:GetInputItem("A") end,
                    set = function(info, value) TradeFill:SetSelectItem("A", tonumber(value)) end,
                    order = 3,
                },
                separatorB = {
                    type = "description",
                    name = " ",
                    order = 4,
                },
                iconB = {
                    type = "description",
                    fontSize = "medium",
                    name = function(info) return TradeFill:GetName("B") end,
                    image = function(info) return TradeFill:SetIconImage("B") end,
                    imageWidth = 32,
                    imageHeight = 32,
                    order = 5,
                },
                itemB = {
                    type = "select",
                    width = 1.5,
                    name = "Item to trade",
                    desc = "Select a item",
                    values = function() return TradeFill:SetOptionItem("B") end,
                    get = function() return TradeFill:GetInputItem("B") end,
                    set = function(info, value) TradeFill:SetSelectItem("B", tonumber(value)) end,
                    order = 6,
                },
                separatorC = {
                    type = "description",
                    name = " ",
                    order = 7,
                },
                toggleBoxB = {
                    type = 'toggle',
                    name = "Only full stacks",
                    desc = "Fill the trade window only with full stacks",
                    get = function(info) return TradeFill.itemdb.profile.stack end,
                    set = function(info, value) TradeFill.itemdb.profile.stack = value end,
                    order = 8,
                },
                toggleBoxC = {
                    type = 'toggle',
                    name = "Hide buttons",
                    desc = "Don't show extra buttons in trade window",
                    get = function(info) return TradeFill.itemdb.profile.button end,
                    set = function(info, value) TradeFill.itemdb.profile.button = value end,
                    order = 9,
                },
                separatorD = {
                    type = "description",
                    name = " ",
                    order = 10,
                },
            },
            inline = true,
        },
        stacks = {
            type = "group",
            name = "Stacks",
            desc = "The number of stacks you want to trade for each class",
            order = 1,
            args = {
            },
        },
        group = {
            type = "group",
            name = "Fill trade",
            desc = "Fill trade window with",
            order = 2,
            args = {
                toggleBoxA = {
                    type = 'toggle',
                    name = "Strangers",
                    desc = "Fill trade window with anyone",
                    get = function() return TradeFill.itemdb.profile.solo end,
                    set = function(info, value) TradeFill.itemdb.profile.solo = value end,
                    order = 0,
                },
                toggleBoxB = {
                    type = 'toggle',
                    name = "Party members",
                    desc = "Fill trade window with party members",
                    get = function() return TradeFill.itemdb.profile.party end,
                    set = function(info, value) TradeFill.itemdb.profile.party = value end,
                    order = 1,
                },
                toggleBoxC = {
                    type = 'toggle',
                    name = "Raid members",
                    desc = "Fill trade window with raid members",
                    get = function() return TradeFill.itemdb.profile.raid end,
                    set = function(info, value) TradeFill.itemdb.profile.raid = value end,
                    order = 2,
                },
                toggleBoxD = {
                    type = 'toggle',
                    name = "Guild members",
                    desc = "Fill trade window with guild members",
                    get = function() return TradeFill.itemdb.profile.guild end,
                    set = function(info, value) TradeFill.itemdb.profile.guild = value end,
                    order = 3,
                },
            },
        },
        exclude = {
            type = "group",
            name = "Exclude",
            desc = "Don't fill the trading window with",
            order = 3,
            args = {
                player = {
                    type = "group",
                    name = "Players",
                    order = 0,
                    args = {
                        inputA = {
                            type = 'input',
                            name = "Player Names",
                            desc = "Don't fill trade window with those players",
                            get = function() return TradeFill.itemdb.profile.players end,
                            set = function(info, value) TradeFill.itemdb.profile.players = value:gsub("\n*$", "") end,
                            pattern = "^[A-Za-z\n]*$",
                            usage = "Player names can only contains letters",
                            multiline = 10,
                            order = 0,
                        },
                    },
                },
                guild = {
                    type = "group",
                    name = "Guilds",
                    order = 1,
                    args = {
                        inputA = {
                            type = 'input',
                            name = "Guild Names",
                            desc = "Don't fill trade window with players from these guilds",
                            get = function() return TradeFill.itemdb.profile.guilds end,
                            set = function(info, value) TradeFill.itemdb.profile.guilds = value:gsub("\n*$", "") end,
                            pattern = "^[A-Za-z\n ]*$",
                            usage = "Guild names can only contains letters",
                            multiline = 10,
                            order = 1,
                        },
                    },
                },
            },
        },
    },
}

function capitalizeFirstLetter(str)
    return str:sub(1, 1):upper() .. str:sub(2):lower()
end

for _, class in pairs(classes) do
    local _, _, _, argbHex = GetClassColor(class)
    options.args.stacks.args[class] = {
        type = "group",
        name = "|c".. argbHex .. capitalizeFirstLetter(class) .. "|r",
        args = {
            iconA = {
                type = "description",
                fontSize = "medium",
                name = function(info) return TradeFill:GetName("A") end,
                image = function(info) return TradeFill:SetIconImage("A") end,
                imageWidth = 16,
                imageHeight = 16,
                order = 0,
            },
            selectedOptionA = {
                type = "select",
                width = 0.75,
                name = "Number of stacks",
                desc = "Choose an amount of stack you want to trade",
                values = function() return TradeFill:SetOptionStack("A") end,
                get = function() return tostring(TradeFill.stacksdb.profile[class].stackNumberA) end,
                set = function(info, value) TradeFill.stacksdb.profile[class].stackNumberA = tonumber(value) TradeFill:UpdateOptionStack(class, "A", tonumber(value)) end,
                order = 1,
            },
            separatorA = {
                type = "description",
                name = " ",
                order = 2,
            },
            iconB = {
                type = "description",
                fontSize = "medium",
                name = function(info) return TradeFill:GetName("B") end,
                image = function(info) return TradeFill:SetIconImage("B") end,
                imageWidth = 16,
                imageHeight = 16,
                order = 3,
            },
            selectedOptionB = {
                type = "select",
                width = 0.75,
                name = "Number of stacks",
                desc = "Choose an amount of stack you want to trade",
                values = function() return TradeFill:SetOptionStack("B") end,
                get = function() return tostring(TradeFill.stacksdb.profile[class].stackNumberB) end,
                set = function(info, value) TradeFill.stacksdb.profile[class].stackNumberB = tonumber(value) TradeFill:UpdateOptionStack(class, "B", tonumber(value)) end,
                order = 4,
            },
        },
    }
end

function itemData(id)

    local itemName, itemLink, _, _, _, _, _, itemStackCount, _, iconTexture = GetItemInfo(id)

    return { ["name"] = itemName, ["link"] = itemLink, ["stack"] = itemStackCount, ["icon"] = iconTexture }
end

function TradeFill:enableDisableClassSetting(value, option)

    if option == "A" then

        if value == 0 then
            for _, class in pairs(classes) do
                options.args.stacks.args[class].args.selectedOptionA.disabled = true
            end
        else
            for _, class in pairs(classes) do
                options.args.stacks.args[class].args.selectedOptionA.disabled = false
            end
        end
    end

    if option == "B" then

        if value == 0 then
            for _, class in pairs(classes) do
                options.args.stacks.args[class].args.selectedOptionB.disabled = true
            end
        else
            for _, class in pairs(classes) do
                options.args.stacks.args[class].args.selectedOptionB.disabled = false
            end
        end
    end
end

function TradeFill:enableDisableButton(value, option)

    if option == "A" then

        if value == 0 or TradeFill.itemdb.profile.button == true then
            ButtonA:Hide()
        else
            ButtonA:Show()
        end
    end

    if option == "B" then

        if value == 0 or TradeFill.itemdb.profile.button == true then
            ButtonB:Hide()
        else
            ButtonB:Show()
        end
    end

    if TradeFill.itemdb.profile.A == 0 and TradeFill.itemdb.profile.B == 0 or TradeFill.itemdb.profile.button == true then
        ClearTrade:Hide()
    else
        ClearTrade:Show()
    end
end

function TradeFill:SetOptionStack(option)

    local selectOption = {}

    if option == "A" then
        for i = 0, NUM_TRADING_SLOTS do
            local itemId = TradeFill.itemdb.profile.itemIdA

            if itemId == 0 then
                selectOption[tostring(i)] = i
            else
                selectOption[tostring(i)] = i
            end
        end
    end

    if option == "B" then
        for i = 0, NUM_TRADING_SLOTS do
            local itemId = TradeFill.itemdb.profile.itemIdB

            if itemId == 0 then
                selectOption[tostring(i)] = i
            else
                selectOption[tostring(i)] = i
            end
        end
    end

    return selectOption
end

function TradeFill:SetOptionItem(option)

    for bag = 0, 4 do
        for slot = 1, C_Container.GetContainerNumSlots(bag) do
            local bagItem = C_Container.GetContainerItemInfo(bag, slot)

            if bagItem ~= nil then
                interfaceIndex = interfaceIndex + 1
                interfaceItem[interfaceIndex] = bagItem
            end
        end
    end

    local selectOption = {}

    selectOption["0"] = "None"

    for i, slot in pairs(interfaceItem) do
        local itemName, itemLink, _, _, _, _, _, itemStackCount, _, iconTexture = GetItemInfo(slot.itemID)

        if TradeFill.itemdb.profile.show == true then
            if slot.isBound == false and slot.stackCount == itemStackCount then
                selectOption[tostring(slot.itemID)] = "|T" .. GetItemIcon(slot.itemID) .. ":0|t " .. slot.hyperlink:gsub("[%[%]]", "")
            end
        else
            if slot.isBound == false then
                selectOption[tostring(slot.itemID)] = "|T" .. GetItemIcon(slot.itemID) .. ":0|t " .. slot.hyperlink:gsub("[%[%]]", "")
            end
        end

        if option == "A" and TradeFill.itemdb.profile.itemIdA > 0 then
            local item = TradeFill.itemdb.profile.itemIdA
            local _, itemLink, _, _, _, _, _, _, _, _ = GetItemInfo(item)

            selectOption[tostring(item)] = "|T" .. GetItemIcon(item) .. ":0|t " .. itemLink:gsub("[%[%]]", "")
        end

        if option == "B" and TradeFill.itemdb.profile.itemIdB > 0 then
            local item = TradeFill.itemdb.profile.itemIdB
            local _, itemLink, _, _, _, _, _, _, _, _ = GetItemInfo(item)

            selectOption[tostring(item)] = "|T" .. GetItemIcon(item) .. ":0|t " .. itemLink:gsub("[%[%]]", "")
        end
    end
    interfaceItem = {}
    interfaceIndex = 0
    return selectOption
end

function TradeFill:UpdateOptionStack(class, option, value)

    local stackA = self.stacksdb.profile[class].stackNumberA
    local stackB = self.stacksdb.profile[class].stackNumberB

    local newValue = NUM_TRADING_SLOTS - value

    if option == "A" then
        if stackA + stackB > NUM_TRADING_SLOTS then

            self.stacksdb.profile[class].stackNumberB = newValue
        end
    end

    if option == "B" then
        if stackA + stackB > NUM_TRADING_SLOTS then

            self.stacksdb.profile[class].stackNumberA = newValue
        end
    end
end

function TradeFill:GetInputItem(option)

    if option == "A" then
        local value = TradeFill.itemdb.profile.itemIdA
        TradeFill:enableDisableClassSetting(value, option)
        TradeFill:enableDisableButton(value, option)
        return tostring(value)
    end

    if option == "B" then
        local value = TradeFill.itemdb.profile.itemIdB
        TradeFill:enableDisableClassSetting(value, option)
        TradeFill:enableDisableButton(value, option)
        return tostring(value)
    end
end

function TradeFill:SetSelectItem(option, value)

    local item = itemData(value)

    if option == "A" then

        ButtonA:SetText(item.name)
        ButtonA:SetPoint("BOTTOM", OptionFrame, "BOTTOMRIGHT", 10 + ButtonA:GetFontString():GetStringWidth() / 2, -19)
        ButtonA:SetSize(ButtonA:GetFontString():GetStringWidth() + 20, 22)

        self.itemdb.profile.itemNameA = item.name
        self.itemdb.profile.itemIdA = value

        TradeFill:enableDisableClassSetting(value, option)
        TradeFill:enableDisableButton(value, option)
    end

    if option == "B" then

        ButtonB:SetText(item.name)
        ButtonB:SetPoint("BOTTOM", OptionFrame, "BOTTOMRIGHT", 10 + ButtonB:GetFontString():GetStringWidth() / 2, -38)
        ButtonB:SetSize(ButtonB:GetFontString():GetStringWidth() + 20, 22)

        self.itemdb.profile.itemNameB = item.name
        self.itemdb.profile.itemIdB = value

        TradeFill:enableDisableClassSetting(value, option)
        TradeFill:enableDisableButton(value, option)
    end
end

function TradeFill:GetName(option)

    if option == "A" then
        local item = itemData(self.itemdb.profile.itemIdA)

        if item.name then
            return item.link:gsub("[%[%]]", "")
        else
            return "Unknown"
        end
    end

    if option == "B" then
        local item = itemData(self.itemdb.profile.itemIdB)

        if item.name then
            return item.link:gsub("[%[%]]", "")
        else
            return "Unknown"
        end
    end
end

function TradeFill:SetIconImage(option)

    if option == "A" then
        local item = itemData(self.itemdb.profile.itemIdA)

        if item.name then
            return item.icon
        else
            return 134400
        end
    end

    if option == "B" then
        local item = itemData(self.itemdb.profile.itemIdB)

        if item.name then
            return item.icon
        else
            return 134400
        end
    end
end

LibStub("AceConfig-3.0"):RegisterOptionsTable("MyAddon", options)
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
AceConfigDialog:AddToBlizOptions("MyAddon", "TradeFill")

local defaultsStacks = {
    profile = {
        ["**"] = { stackNumberA = 0, stackNumberB = 0 }
    }
}

local defaultsItems = {
    profile = {
        itemIdA = 0, itemIdB = 0, show = false, stack = false, button = false, solo = false, party = false, raid = false, guild = false, players = "", guilds = "",
    }
}

-----------------------------------------------
-----------------------------------------------

function playerStatus()
    local flag = false
    local playerNames = TradeFill.itemdb.profile.players

    for name in string.gmatch(playerNames, "[^\n]+") do
        if string.lower(name) == string.lower(GetUnitName("npc")) then
            flag = true
        end
    end
    return flag
end

function guildStatus()
    local flag = false
    local guildNames = TradeFill.itemdb.profile.guilds

    if GetGuildInfo("npc") then
        for guild in string.gmatch(guildNames, "[^\n]+") do
            if string.lower(guild) == string.lower(GetGuildInfo("npc")) then
                flag = true
            end
        end
    end
    return flag
end

function groupStatus()

    if IsInRaid() then
        return { ["status"] = "raid" }
    elseif IsInGroup() then
        return { ["status"] = "party" }
    else
        return { ["status"] = "solo" }
    end
end

function tradePosition()
    for i = 1, MAX_TRADABLE_ITEMS do
        if GetTradePlayerItemLink(i) == nil then
            return i
        end
    end
end

function tradeFill(item, position)
    local _, itemLink = GetItemInfo(item)

    if next(inventory) == nil then
        print(color .. " Out of stock " .. itemLink .. " |T" .. GetItemIcon(item) .. ":0|t (item not found)")
        return
    end

    if position == nil then
        print(color .. " Trade window is full!|r")
        return
    else
        for i = 1, inventoryIndex do
            local locked = C_Container.GetContainerItemInfo(inventory[i].bag, inventory[i].slot)
            local stackCount = itemData(inventory[i].item)

            if inventory[i].bound == true then
                print(color .. " Item is Soulbound " .. itemLink .. " |T" .. GetItemIcon(item) .. ":0|t (can't trade that)")
                return
            end

            if TradeFill.itemdb.profile.stack == false then
                if inventory[i].item == item and locked.isLocked == false then
                    ClearCursor()
                    C_Container.PickupContainerItem(inventory[i].bag, inventory[i].slot)
                    ClickTradeButton(position)
                    break
                end

                if i >= inventoryIndex then
                    print(color .. " Out of stock " .. itemLink .. " |T" .. GetItemIcon(item) .. ":0|t (any size stacks)")
                    return
                end
            end

            if TradeFill.itemdb.profile.stack == true then
                if inventory[i].item == item and locked.isLocked == false and inventory[i].count == stackCount.stack then
                    ClearCursor()
                    C_Container.PickupContainerItem(inventory[i].bag, inventory[i].slot)
                    ClickTradeButton(position)
                    break
                end
                if i >= inventoryIndex then
                    print(color .. " Out of stock " .. itemLink .. " |T" .. GetItemIcon(item) .. ":0|t (only full stacks)")
                    return
                end
            end
        end
    end
end

function start()
    local itemA = TradeFill.itemdb.profile.itemIdA
    local itemB = TradeFill.itemdb.profile.itemIdB

    local _, ClassName, _ = UnitClass("npc")

    for bag = 0, 4 do
        for slot = 1, C_Container.GetContainerNumSlots(bag) do
            local bagItem = C_Container.GetContainerItemInfo(bag, slot)

            if bagItem ~= nil then
                if bagItem.itemID == itemA or bagItem.itemID == itemB then
                    inventoryIndex = inventoryIndex + 1
                    inventory[inventoryIndex] = {
                        ["bag"] = bag,
                        ["slot"] = slot,
                        ["item"] = bagItem.itemID,
                        ["bound"] = bagItem.isBound,
                        ["count"] = bagItem.stackCount,
                        ["link"] = bagItem.hyperlink
                    }
                end
            end
        end
    end

    if playerStatus() then
        print(color .. " Don't fill trade window with |cFF00FF00<".. UnitName("npc") .. ">|r this player is on the exclude list")
        return
    end

    if guildStatus() then
        print(color .. " Don't fill trade window with |cFF00FF00<".. GetGuildInfo("npc") .. ">|r this guild is on the exclude list")
        return
    end

    local group = groupStatus()

    if group.status == "party" or group.status == "raid" then

        local numGroupMembers = GetNumGroupMembers()

        if numGroupMembers > 0 then
            for i = 1, numGroupMembers do
                local unitName = UnitName(group.status .. i)

                if unitName == UnitName("npc") then
                    if (TradeFill.itemdb.profile.solo == true and group.status == "solo") or (TradeFill.itemdb.profile.party == true and group.status == "party") or (TradeFill.itemdb.profile.raid == true and group.status == "raid") then

                        if itemA > 0 then
                            for i = 1, TradeFill.stacksdb.profile[ClassName].stackNumberA do
                                tradeFill(itemA, tradePosition())
                            end
                        end

                        if itemB > 0 then
                            for i = 1, TradeFill.stacksdb.profile[ClassName].stackNumberB do
                                tradeFill(itemB, tradePosition())
                            end
                        end
                    end
                    break
                end
            end
        end
    end

    if TradeFill.itemdb.profile.solo == true and group.status == "solo" or GetGuildInfo("npc") == GetGuildInfo("player") then
        if itemA > 0 then
            for i = 1, TradeFill.stacksdb.profile[ClassName].stackNumberA do
                tradeFill(itemA, tradePosition())
            end
        end

        if itemB > 0 then
            for i = 1, TradeFill.stacksdb.profile[ClassName].stackNumberB do
                tradeFill(itemB, tradePosition())
            end
        end
    end
end

function close()
    inventoryIndex = 0
    inventory = {}
end

function TradeFill:OnInitialize()
    self.stacksdb = LibStub("AceDB-3.0"):New("StacksDB", defaultsStacks)
    self.itemdb = LibStub("AceDB-3.0"):New("ItemDB", defaultsItems)

    local faction = UnitFactionGroup("player")

    if faction == "Alliance" then
        options.args.stacks.args["SHAMAN"].hidden = true
    elseif faction == "Horde" then
        options.args.stacks.args["PALADIN"].hidden = true
    end

    OptionFrame:SetWidth(200)
    OptionFrame:SetHeight(600)
    OptionFrame:SetPoint("BOTTOMRIGHT", TradeFrame, "BOTTOMRIGHT", 0, 0)

    ButtonA:SetText(TradeFill.itemdb.profile.itemNameA)
    ButtonA:SetPoint("BOTTOM", OptionFrame, "BOTTOMRIGHT", 10 + ButtonA:GetFontString():GetStringWidth() / 2, -19)
    ButtonA:SetSize(ButtonA:GetFontString():GetStringWidth() + 20, 22)
    local function ButtonItemA()
        tradeFill(TradeFill.itemdb.profile.itemIdA, tradePosition())
    end
    ButtonA:SetScript("OnClick", ButtonItemA)
    TradeFill:enableDisableButton(TradeFill.itemdb.profile.itemIdA, "A")

    ButtonB:SetText(TradeFill.itemdb.profile.itemNameB)
    ButtonB:SetPoint("BOTTOM", OptionFrame, "BOTTOMRIGHT", 10 + ButtonB:GetFontString():GetStringWidth() / 2, -38)
    ButtonB:SetSize(ButtonB:GetFontString():GetStringWidth() + 20, 22)
    local function ButtonItemB()
        tradeFill(TradeFill.itemdb.profile.itemIdB, tradePosition())
    end
    ButtonB:SetScript("OnClick", ButtonItemB)
    TradeFill:enableDisableButton(TradeFill.itemdb.profile.itemIdB, "B")

    ClearTrade:SetText("Clear")
    ClearTrade:SetPoint("BOTTOMRIGHT", OptionFrame, "BOTTOMRIGHT", 73, 5)
    ClearTrade:SetSize(75, 22)
    local function ButtonClear()
        for i = 1, MAX_TRADABLE_ITEMS do
            ClearCursor()
            ClickTradeButton(i)
        end
        ClearCursor()
    end
    ClearTrade:SetScript("OnClick", ButtonClear)
end

TradeFill:RegisterEvent("TRADE_SHOW", start)
TradeFill:RegisterEvent("TRADE_CLOSED", close)